import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { DndProvider } from 'react-dnd'
import HTML5Backend from 'react-dnd-html5-backend'
import { Button, Card, Collapse, notification, Spin, Col } from 'antd'
import { RedoOutlined } from '@ant-design/icons'

import Api from '@/api'
import Utils from '@/utils/utils.js'
import { updateCommonTable } from '@/utils/utils-update.js'

import asyncComponent from '@/utils/asyncComponent'
import SearchComponent from '@/templates/sharecomponent/searchcomponent'
import ActionComponent from '@/templates/sharecomponent/actioncomponent'
import ColumnComponent from '@/templates/sharecomponent/columncomponent'

import MenuForm from './menuform'
import Source from './source'
import './index.scss'

const { Panel } = Collapse

const UrlFieldComponent = asyncComponent(() => import('@/menu/urlfieldcomponent'))
const UpdateTable = asyncComponent(() => import('./updatetable'))
const Unattended = asyncComponent(() => import('@/templates/zshare/unattended'))
const SettingComponent = asyncComponent(() => import('@/templates/sharecomponent/settingcomponent'))
const TableComponent = asyncComponent(() => import('@/templates/sharecomponent/tablecomponent'))
const ChartGroupComponent = asyncComponent(() => import('@/templates/sharecomponent/chartgroupcomponent'))
const ChartComponent = asyncComponent(() => import('@/templates/sharecomponent/chartcomponent'))
const CardComponent = asyncComponent(() => import('@/templates/sharecomponent/cardcomponent'))
const TabsComponent = asyncComponent(() => import('@/templates/sharecomponent/tabscomponent'))

class ComTableConfig extends Component {
  static propTpyes = {
    menu: PropTypes.any,
    reloadmenu: PropTypes.func,
    handleView: PropTypes.func
  }

  state = {
    config: null,            // 页面配置
    formlist: null,          // 搜索条件、按钮、显示列表单字段
    menuloading: false,      // 菜单保存中
    menucloseloading: false, // 菜单关闭时，选择保存
    loading: false,          // 加载中，页面spin
    closeVisible: false,     // 关闭模态框
    originMenu: null,        // 原始菜单
    originActions: null,     // 原始按钮信息，使用已有用户模板
    delActions: [],          // 删除按钮列表
    copyActions: [],         // 复制按钮组
    tabviews: [],            // 所有标签页
    activeKey: '0',          // 默认展开基本信息
    chartview: null,         // 当前视图
    openEdition: '',         // 编辑版本标记，防止多人操作
  }

  /**
   * @description 数据预处理
   * 1、设置页面配置信息，新建或无配置信息时（切换模板后无配置信息），使用模板默认配置
   * 2、设置操作类型、原始菜单信息（每次保存后重置）、已使用表及基本信息表单
   */
  UNSAFE_componentWillMount () {
    const { menu } = this.props
    let _LongParam = menu.LongParam
    let _config = ''

    if (!_LongParam) {
      _config = fromJS(Source.baseConfig).toJS()
      _config.isAdd = true
    } else {
      _config = _LongParam
    }

    // 页面配置中保留菜单信息，只用于数据传递
    _config.ParentId = menu.ParentId
    _config.fstMenuId = menu.fstMenuId
    _config.MenuName = menu.MenuName || ''
    _config.MenuNo = menu.MenuNo || ''
    _config.OpenType = menu.PageParam ? menu.PageParam.OpenType : ''
    _config.easyCode = _config.easyCode || ''
    _config.uuid = menu.MenuID || ''

    // 版本兼容
    _config = updateCommonTable(_config)
    
    let _oriActions = []
    if (_config.type === 'user') {
      _config.action = _config.action.map(item => {
        let uuid = Utils.getuuid()

        if (item.linkTab) {
          item.linkTab = ''
        }

        if (item.OpenType === 'pop' || item.execMode === 'pop') { // 含有子配置项的按钮(表单)
          _oriActions.push({
            prebtn: fromJS(item).toJS(),
            curuuid: uuid,
            Template: 'Modal'
          })
        }

        item.uuid = uuid
        return item
      })

      // 重置标签ID
      _config.tabgroups.forEach(group => {
        group.sublist = group.sublist.map(tab => {
          tab.uuid = Utils.getuuid()
          
          if (tab.linkTab) {
            tab.linkTab = ''
          }

          return tab
        })
      })
    }

    this.setState({
      chartview: _config.charts ? _config.charts[0].uuid : '',
      config: _config,
      openEdition: menu.open_edition || '',
      activeKey: menu.activeKey || '0',
      originActions: _oriActions,
      originMenu: fromJS(_config).toJS()
    })
  }

  /**
   * @description 加载完成后, 获取所有标签页信息
   */
  componentDidMount () {
    this.reloadTab(false)

    document.onkeydown = (event) => {
      let e = event || window.event
      let keyCode = e.keyCode || e.which || e.charCode
      let preKey = ''

      if (e.ctrlKey) {
        preKey = 'ctrl'
      }
      if (e.shiftKey) {
        preKey = 'shift'
      } else if (e.altKey) {
        preKey = 'alt'
      }
      
      if (!preKey || !keyCode) return
      
      let _shortcut = `${preKey}+${keyCode}`

      if (_shortcut === 'ctrl+83') {
        let modals = document.querySelectorAll('.mk-pop-modal')
        let msg = null
        for (let i = 0; i < modals.length; i++) {
          if (msg) {
            break
          }

          let node = modals[i].querySelector('.mk-com-name')

          if (node) {
            msg = node.innerText
          }
        }
        if (msg) {
          notification.warning({
            top: 92,
            message: '请保存' + msg,
            duration: 5
          })
          return false
        }

        let node = document.getElementById('save-config')
        if (node && node.click) {
          node.click()
        }
        return false
      }
    }
  }

  /**
   * @description 组件销毁，清除state更新
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    document.onkeydown = () => {}
  }

  /**
   * @description 加载或刷新标签信息
   */
  reloadTab = (type) => {
    this.setState({
      loading: type,
      tabviews: []
    })
    Api.getCloudConfig({func: 'sPC_Get_UserTemp', TypeCharTwo: 'tab'}).then(res => {
      if (res.status) {
        this.setState({
          loading: false,
          tabviews: res.UserTemp.map(temp => {
            return {
              uuid: temp.MenuID,
              value: temp.MenuID,
              text: temp.MenuName,
              type: temp.Template,
              MenuNo: temp.MenuNo
            }
          })
        })

        if (type) {
          notification.success({
            top: 92,
            message: '刷新成功。',
            duration: 2
          })
        }
      } else {
        this.setState({
          loading: false
        })
        notification.warning({
          top: 92,
          message: res.message,
          duration: 5
        })
      }
    })
  }

  getFuncNames = (config) => {
    let funcNames = []
    let tableNames = []

    if (config.setting.tableName) {
      tableNames.push(config.setting.tableName)
    }
    if (config.setting.innerFunc) {
      funcNames.push({func: config.setting.innerFunc, label: config.MenuName || ''})
    }
    if (config.setting.outerFunc) {
      funcNames.push({func: config.setting.outerFunc, label: config.MenuName || ''})
    }

    config.action.forEach(item => {
      let tablename = item.OpenType === 'excelIn' ? (item.sheet || '') : (item.sql || '')

      if (item.OpenType === 'excelOut' && item.intertype === 'system') {
        tablename = config.setting.tableName || ''
      }

      if (['pop', 'prompt', 'exec', 'excelIn', 'excelOut', 'funcbutton'].includes(item.OpenType)) {
        if (tablename) {
          tableNames.push(tablename)
        }
        if (item.innerFunc) {
          funcNames.push({func: item.innerFunc, label: item.label || ''})
        }
        if (item.callbackFunc) {
          funcNames.push({func: item.callbackFunc, label: item.label || ''})
        }
      }
    })

    tableNames = Array.from(new Set(tableNames))

    return {
      func: funcNames,
      table: tableNames
    }
  }

  /**
   * @description 三级菜单保存
   */
  // submitConfig = () => {
  //   const { menu } = this.props
  //   const { delActions, openEdition } = this.state

  //   let _config = fromJS(this.state.config).toJS()

  //   // 基本信息验证
  //   if (!_config.fstMenuId || !_config.ParentId || !_config.MenuName || !_config.MenuNo) {
  //     notification.warning({
  //       top: 92,
  //       message: '请完善菜单基本信息！',
  //       duration: 5
  //     })
  //     this.setState({activeKey: '0'})
  //     return
  //   }

  //   // 新建菜单，清除默认项
  //   if (_config.isAdd) {
  //     _config.search = _config.search.filter(item => !item.origin)
  //     _config.action = _config.action.filter(item => !item.origin)
  //     _config.columns = _config.columns.filter(item => !item.origin)
  //     _config.tabgroups[0].sublist = _config.tabgroups[0].sublist.filter(item => !item.origin)
  //   }

  //   // 使用已有菜单时，默认添加关联标签id
  //   if (_config.type === 'user') {
  //     _config.action = _config.action.map(item => {
  //       if (item.OpenType === 'popview' && !item.linkTab) {
  //         item.linkTab = Utils.getuuid()
  //       }
  //       return item
  //     })
  
  //     _config.tabgroups.forEach(group => {
  //       group.sublist = group.sublist.map(tab => {
  //         if (!tab.linkTab) {
  //           tab.linkTab = Utils.getuuid()
  //         }
  //         return tab
  //       })
  //     })
  //   }

  //   // 按钮不存在时，去掉绑定的双击按钮
  //   if (_config.setting.doubleClick && _config.action.findIndex((item) => item.uuid === _config.setting.doubleClick) === -1) {
  //     _config.setting.doubleClick = ''
  //   }

  //   // 未设置数据源或标签不合法时，启用状态为false
  //   let vresult = this.verifyconfig(_config)
  //   if (vresult !== true) {
  //     _config.enabled = false
  //   }

  //   if (this.state.closeVisible) { // 显示关闭对话框时，模态框中保存按钮，显示保存中状态
  //     this.setState({
  //       menucloseloading: true
  //     })
  //   } else {
  //     this.setState({
  //       menuloading: true
  //     })
  //   }

  //   let _LongParam = ''

  //   // 保存时删除配置类型，system 、user
  //   delete _config.type
  //   delete _config.isAdd

  //   try {
  //     _LongParam = window.btoa(window.encodeURIComponent(JSON.stringify(_config)))
  //   } catch (e) {
  //     notification.warning({
  //       top: 92,
  //       message: '编译错误',
  //       duration: 5
  //     })
  //     this.setState({
  //       menucloseloading: false,
  //       menuloading: false
  //     })
  //     return
  //   }

  //   let _sort = 0
  //   // let btntabs = []

  //   let btnParam = {             // 添加菜单按钮
  //     func: 'sPC_Button_AddUpt',
  //     Type: 40,                  // 添加菜单下的按钮type为40，按钮下的按钮type为60
  //     ParentID: menu.MenuID,
  //     MenuNo: _config.MenuNo,
  //     Template: _config.Template || '',
  //     PageParam: '',
  //     LongParam: '',
  //     LText: []
  //   }

  //   _config.action.forEach(item => {
  //     if (item.hidden === 'true') return
  //     _sort++
  //     // if (item.OpenType === 'popview') {
  //     //   btntabs.push({
  //     //     uuid: item.uuid,
  //     //     linkTab: item.linkTab,
  //     //     label: item.label,
  //     //     sort: _sort
  //     //   })
  //     // }
      
  //     btnParam.LText.push(`select '${item.uuid}' as menuid, '${item.label}' as menuname, '${_sort * 10}' as Sort`)
  //   })

  //   btnParam.LText = btnParam.LText.join(' union all ')
  //   btnParam.LText = Utils.formatOptions(btnParam.LText)
  //   btnParam.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
  //   btnParam.secretkey = Utils.encrypt(btnParam.LText, btnParam.timestamp)
    
  //   // let tabParam = { // 添加菜单tab页
  //   //   func: 'sPC_sMenusTab_AddUpt',
  //   //   MenuID: menu.MenuID
  //   // }

  //   // let _LText = []

  //   // btntabs.forEach(item => {
  //   //   _LText.push(`select '${item.uuid}' as MenuID ,'${item.linkTab}' as Tabid,'${item.label}' as TabName ,'${item.sort * 10}' as Sort`)
  //   // })
  //   // _config.tabgroups.forEach(group => {
  //   //   group.sublist.forEach(item => {
  //   //     _sort++
  //   //     _LText.push(`select '${menu.MenuID}' as MenuID ,'${item.linkTab}' as Tabid,'${item.label}' as TabName ,'${_sort * 10}' as Sort`)
  //   //   })
  //   // })

  //   // _LText = _LText.join(' union all ')

  //   // 清空菜单下关联的标签
  //   // if (!_LText) {
  //   //   _LText = `select '${menu.MenuID}' as MenuID ,'' as Tabid,'' as TabName ,'0' as Sort`
  //   // }

  //   // tabParam.LText = Utils.formatOptions(_LText)
  //   // tabParam.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
  //   // tabParam.secretkey = Utils.encrypt(tabParam.LText, tabParam.timestamp)

  //   let _vals = this.getFuncNames(_config)

  //   let param = {
  //     func: 'sPC_TrdMenu_AddUpt',
  //     FstID: _config.fstMenuId,
  //     SndID: _config.ParentId,
  //     ParentID: _config.ParentId,
  //     MenuID: menu.MenuID,
  //     MenuNo: _config.MenuNo,
  //     EasyCode: _config.easyCode || '',
  //     Template: _config.Template || '',
  //     MenuName: _config.MenuName,
  //     PageParam: JSON.stringify({...menu.PageParam, Template: _config.Template, OpenType: _config.OpenType, hidden: _config.hidden || 'false'}),
  //     LongParam: _LongParam,
  //     LText: _vals.func.map(item => `select '${menu.MenuID}' as MenuID,'${item.func}' as ProcName,'${item.label}' as MenuName`),
  //     LTexttb: _vals.table.map(item => `select '${menu.MenuID}' as MenuID,'${item}' as tbName`)
  //   }

  //   if (menu.menuSort) { // 菜单新建时设置排序
  //     param.Sort = menu.menuSort
  //   }

  //   param.LText = param.LText.join(' union all ')
  //   param.LText = Utils.formatOptions(param.LText)
  //   param.LTexttb = param.LTexttb.join(' union all ')
  //   param.LTexttb = Utils.formatOptions(param.LTexttb)
  //   param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
  //   param.secretkey = Utils.encrypt(param.LText, param.timestamp)

  //   if (openEdition) { // 版本管理
  //     param.open_edition = openEdition
  //   }

  //   setTimeout(() => {
  //     // 有按钮或标签删除时，先进行删除操作
  //     // 删除成功后，保存页面配置
  //     new Promise(resolve => {
  //       if (delActions.length > 0) {
  //         let deffers = delActions.map(item => {
  //           let _param = {
  //             func: 'sPC_MainMenu_Del',
  //             MenuID: item.card ? item.card.uuid : item.uuid
  //           }

  //           if (item.type === 'action') {
  //             let _ParentParam = null

  //             try {
  //               _ParentParam = window.btoa(window.encodeURIComponent(JSON.stringify(item.card)))
  //             } catch (e) {
  //               console.warn('Stringify Failure')
  //               _ParentParam = null
  //             }

  //             if (_ParentParam) { // 删除按钮时，保存按钮配置信息，用于恢复按钮
  //               _param.ParentParam = _ParentParam
  //             }
  //           }

  //           return new Promise(resolve => {
  //             Api.getCloudConfig(_param).then(response => {
  //               resolve(response)
  //             })
  //           })
  //         })
  //         Promise.all(deffers).then(result => {
  //           let error = null
  //           result.forEach(response => {
  //             if (!response.status) {
  //               error = response
  //             }
  //           })

  //           if (error) {
  //             this.setState({
  //               menuloading: false,
  //               menucloseloading: false
  //             })
  //             notification.warning({
  //               top: 92,
  //               message: error.message,
  //               duration: 5
  //             })
  //             resolve(false)
  //           } else {
  //             this.setState({
  //               delActions: []
  //             })
  //             resolve(true)
  //           }
  //         })
  //       } else if (delActions.length === 0) {
  //         resolve(true)
  //       }
  //     }).then(resp => {
  //       if (resp === false) return

  //       return true
  //     }).then(res => {
  //       if (res === true || res === false) return res

  //       let msg = res.filter(Boolean)[0]
  //       if (msg) {
  //         notification.warning({
  //           top: 92,
  //           message: msg,
  //           duration: 5
  //         })
  //         return false
  //       } else {
  //         return true
  //       }
  //     }).then(resp => {
  //       if (resp === false) return
  //       Api.getCloudConfig(param).then(response => {
  //         if (response.status) {
  //           this.setState({
  //             config: _config,
  //             openEdition: response.open_edition || '',
  //             originMenu: fromJS(_config).toJS()
  //           })

  //           this.submitAction(btnParam)
  //         } else {
  //           this.setState({
  //             menuloading: false,
  //             menucloseloading: false
  //           })
  //           notification.warning({
  //             top: 92,
  //             message: response.message,
  //             duration: 5
  //           })
  //         }
  //       })
  //     })
  //   }, +sessionStorage.getItem('mkDelay'))
  // }

  /**
   * @description 保存或修改菜单按钮集
   */
  // submitAction = (btnParam) => {
  //   const { config } = this.state

  //   new Promise(resolve => {
  //     if (btnParam.LText) {
  //       Api.getCloudConfig(btnParam).then(result => {
  //         if (result.status) {
  //           this.setState({ // 保存成功后清空复制列表
  //             copyActions: []
  //           })
  //           resolve(result)
  //         } else {
  //           notification.warning({
  //             top: 92,
  //             message: result.message,
  //             duration: 5
  //           })
  //           resolve(false)
  //         }
  //       })
  //     } else {
  //       resolve(true)
  //     }
  //   }).then(response => {
  //     if (response === false) return response

  //     if (!this.state.originActions || this.state.originActions.length === 0) return 'true'

  //     let oriActions = []
  //     this.state.originActions.forEach(item => {
  //       let curBtn = config.action.filter(cell => item.curuuid === cell.uuid)[0] // 查看初始化按钮是否存在
  //       if (!curBtn) return
  //       if (curBtn.OpenType !== item.prebtn.OpenType) return
  //       if (curBtn.OpenType === 'funcbutton' && curBtn.execMode !== 'pop') return

  //       oriActions.push({
  //         prebtn: item.prebtn,
  //         curBtn: curBtn
  //       })
  //     })

  //     if (oriActions.length === 0) return 'true'

  //     oriActions.forEach(action => {
  //       if (!action.prebtn || !action.prebtn.uuid) return

  //       Api.getCloudConfig({
  //         func: 'sPC_Get_LongParam',
  //         MenuID: action.prebtn.uuid
  //       }).then(result => {
  //         if (result.status && result.LongParam) {
  //           let _temp = ''

  //           let _subconfig = ''
  //           try {
  //             _subconfig = JSON.parse(window.decodeURIComponent(window.atob(result.LongParam)))
  //             _temp = _subconfig.type
  //           } catch (e) {
  //             console.warn('Parse Failure')
  //             _subconfig = ''
  //           }

  //           if (!_subconfig) return

  //           let param = {
  //             func: 'sPC_ButtonParam_AddUpt',
  //             ParentID: this.props.menu.MenuID,
  //             MenuID: action.curBtn.uuid,
  //             MenuNo: config.MenuNo,
  //             Template: _temp,
  //             MenuName: action.curBtn.label,
  //             PageParam: JSON.stringify({Template: _temp}),
  //             LongParam: result.LongParam
  //           }
  //           Api.getCloudConfig(param).then(() => {})
  //         }
  //       })
  //     })
  //     return 'true'
  //   }).then(response => {
  //     if (response === 'true') {
  //       notification.success({
  //         top: 92,
  //         message: '保存成功',
  //         duration: 2
  //       })
  //       if (this.state.closeVisible) {
  //         this.props.handleView()
  //       } else {
  //         this.setState({
  //           originActions: [],
  //           menuloading: false,
  //           menucloseloading: false
  //         })
  //       }
  //       this.props.reloadmenu()
  //     } else {
  //       this.setState({
  //         menuloading: false,
  //         menucloseloading: false
  //       })
  //     }
  //   })
  // }

  /**
   * @description 点击返回时，判断配置保存状态
   */
  cancelConfig = () => {
    // const { config, originMenu } = this.state

    // let that = this

    // if (config.isAdd) {
    //   confirm({
    //     content: '菜单尚未提交，确定放弃保存吗？',
    //     onOk() {
    //       that.props.handleView()
    //     },
    //     onCancel() {}
    //   })
    // } else {
    //   if (!is(fromJS(originMenu), fromJS(config))) {
    //     this.setState({
    //       closeVisible: true
    //     })
    //   } else {
    //     this.props.handleView()
    //   }
    // }

    this.props.handleView()
  }

  /**
   * @description 设置可配置按钮
   */
  setSubConfig = (item, type) => {
    const { menu } = this.props
    const { config, originMenu, activeKey, openEdition } = this.state

    if (config.isAdd) { // 新建菜单，提示菜单尚未保存
      notification.warning({
        top: 92,
        message: '菜单尚未保存，请先保存菜单配置！',
        duration: 5
      })
    } else {
      if (!is(fromJS(originMenu), fromJS(config))) { // 菜单信息变化时，提示保存
        notification.warning({
          top: 92,
          message: '菜单配置已修改，请保存！',
          duration: 5
        })
        return
      }

      let submenu = menu.fstMenuList.filter(_menu => _menu.MenuID === config.fstMenuId)[0]

      let _Menu = {
        ...menu,
        LongParam: config,
        PageParam: {...menu.PageParam, Template: config.Template, OpenType: config.OpenType},
        MenuName: config.MenuName,
        MenuNo: config.MenuNo,
        ParentId: config.ParentId,
        fstMenuId: config.fstMenuId,
        supMenuList: submenu ? submenu.children : []
      }

      // 菜单信息验证通过后，跳转子配置页面
      let _view = ''
      let uuid = item.uuid
      let isbutton = true
      let _btnTab = null
      
      if (type === 'button' && (item.OpenType === 'pop' || item.execMode === 'pop')) {
        _view = 'Modal'      // 表单页面
      } else if (type === 'button' && item.OpenType === 'tab') {
        _view = 'FormTab'    // 表单标签页模板
        _btnTab = item
      } else if (type === 'button' && item.OpenType === 'popview') {
        _view = 'SubTable'   // 新弹窗标签模板 tabType 属性已去除
        uuid = item.linkTab
        isbutton = false
      } else if (type === 'tab') {
        _view = 'SubTable'   // 标签模板
        uuid = item.linkTab
        isbutton = false
      }

      _Menu.activeKey = activeKey       // 保存当前打开页签
      _Menu.open_edition = openEdition  // 更新版本号

      let param = {
        editMenu: _Menu,
        editTab: !isbutton ? item : '',
        tabConfig: null,
        editSubTab: null,
        subTabConfig: null,
        btnTab: _btnTab,
        btnTabConfig: null,
        editAction: isbutton ? item : '',
        subConfig: '',
        tabview: _view
      }
      
      // 当子表使用主页搜索条件时，将主页搜索向下传递
      if (param.editTab && param.editTab.searchPass === 'true') {
        param.editTab.mainsearch = fromJS(config.search).toJS()
      }

      this.setState({
        loading: true
      })

      Api.getCloudConfig({
        func: 'sPC_Get_LongParam',
        MenuID: uuid
      }).then(res => {
        if (res.status) {
          this.setState({
            loading: false
          })
          let _LongParam = ''
          if (res.LongParam) {
            try {
              _LongParam = JSON.parse(window.decodeURIComponent(window.atob(res.LongParam)))
            } catch (e) {
              console.warn('Parse Failure')
              _LongParam = ''
            }
          }

          if (_LongParam && param.tabview === 'Modal' && _LongParam.type === 'Modal') {
            param.subConfig = _LongParam
          } else if (_LongParam && param.tabview === 'FormTab' && _LongParam.type === 'FormTab') {
            param.subConfig = _LongParam
          } else if (_LongParam && param.tabview === 'SubTable' && _LongParam.Template === 'SubTable') {
            param.subConfig = _LongParam
          }
          
          if (param.editTab) {
            param.editTab.open_edition = res.open_edition || ''
          } else if (param.editAction) {
            param.editAction.open_edition = res.open_edition || ''
          } else if (param.btnTab) {
            param.btnTab.open_edition = res.open_edition || ''
          }

          this.props.handleView(param)
        } else {
          this.setState({
            loading: false
          })
          notification.warning({
            top: 92,
            message: res.message,
            duration: 5
          })
        }
      })
    }
  }

  /**
   * @description 页面启用停止切换
   */
  onEnabledChange = () => {
    const { config } = this.state

    let _enabled = !config.enabled
    let result = this.verifyconfig(config)
    if (_enabled && result !== true) {
      notification.warning({
        top: 92,
        message: result,
        duration: 5
      })
      return
    }

    this.setState({
      config: {...config, enabled: _enabled}
    })
  }

  /**
   * @description 校验配置信息的合法性
   */
  verifyconfig = (config) => {
    let tabinvalid = true
    if (config.tabgroups.length > 1) {
      config.tabgroups.forEach(group => {
        if (group.sublist.length === 0) {
          tabinvalid = false
        }
      })
    }

    let hasKey = false
    let chartcols = []
    config.columns.forEach(col => {
      if (col.field) {
        chartcols.push(col.field)
      }
      if (config.setting.primaryKey === col.field) {
        hasKey = true
      }
    })

    let chartError = ''
    config.charts && config.charts.forEach((chart, index) => {
      if (chartError) return
      if (chart.Hide === 'true') return
      if (!['line', 'bar', 'pie'].includes(chart.chartType)) return

      if (!chart.Xaxis) {
        chartError = `图表${chart.title ? '《' + chart.title + '》' : index + 1}坐标轴字段尚未设置，不可启用！`
      } else if (['line', 'bar'].includes(chart.chartType) && chart.datatype !== 'statistics' && (!chart.Yaxis || chart.Yaxis.length === 0)) { // query 查询数据
        chartError = `图表${chart.title ? '《' + chart.title + '》' : index + 1}坐标轴字段尚未设置，不可启用！`
      } else if (['line', 'bar'].includes(chart.chartType) && chart.datatype === 'statistics' && (!chart.InfoType || !chart.InfoValue)) { // statistics 统计数据
        chartError = `图表${chart.title ? '《' + chart.title + '》' : index + 1}坐标轴字段尚未设置，不可启用！`
      } else if (chart.chartType === 'pie' && !chart.Yaxis) {
        chartError = `图表${chart.title ? '《' + chart.title + '》' : index + 1}坐标轴字段尚未设置，不可启用！`
      } else if (!chartcols.includes(chart.Xaxis)) {
        chartError = `图表${chart.title ? '《' + chart.title + '》' : index + 1}坐标轴字段在显示列中不存在，不可启用！`
      } else if (chart.chartType === 'pie' && !chartcols.includes(chart.Yaxis)) {
        chartError = `图表${chart.title ? '《' + chart.title + '》' : index + 1}坐标轴字段在显示列中不存在，不可启用！`
      } else if (['line', 'bar'].includes(chart.chartType) && chart.datatype === 'statistics' && (!chartcols.includes(chart.InfoType) || !chartcols.includes(chart.InfoValue))) { // statistics 统计数据
        chartError = `图表${chart.title ? '《' + chart.title + '》' : index + 1}坐标轴字段在显示列中不存在，不可启用！`
      } else if (['line', 'bar'].includes(chart.chartType) && chart.datatype !== 'statistics' && chart.Yaxis.filter(yaxis => !chartcols.includes(yaxis)).length > 0) {
        chartError = `图表${chart.title ? '《' + chart.title + '》' : index + 1}坐标轴字段在显示列中不存在，不可启用！`
      }
    })
    
    config.action && config.action.forEach((btn) => {
      if (btn.intertype === 'custom' && btn.callbackType === 'script' && (!btn.verify || !btn.verify.cbScripts || !btn.verify.cbScripts.filter(item => item.status !== 'false').length === 0)) {
        notification.warning({
          top: 92,
          message: `按钮《${btn.label}》未设置回调脚本， 将不会生效！`,
          duration: 5
        })
      }
    })

    if (config.setting.interType === 'system' && config.setting.default === 'false' && config.setting.scripts && config.setting.scripts.filter(item => item.status !== 'false').length === 0) {
      return '数据源中不执行默认sql，且未添加自定义脚本，不可启用！'
    } else if (config.setting.interType === 'custom' && config.setting.procMode !== 'inner' && config.setting.preScripts && config.setting.preScripts.filter(item => item.status !== 'false').length === 0) {
      return '数据源未设置前置脚本，不可启用！'
    } else if (config.setting.interType === 'custom' && config.setting.callbackType === 'script' && config.setting.cbScripts && config.setting.cbScripts.filter(item => item.status !== 'false').length === 0) {
      return '数据源未设置回调脚本，不可启用！'
    } else if (!config.setting.primaryKey) {
      return '菜单尚未设置主键，不可启用！'
    } else if (config.columns.length === 0) {
      return '菜单尚未设置显示列，不可启用！'
    } else if (!hasKey) {
      return '显示列中不存在主键字段，不可启用！'
    } else if (!tabinvalid) {
      return '菜单标签页设置错误（存在多行标签时，行标签不可为空）！'
    } else if (chartError) {
      return chartError
    } else {
      return true
    }
  }

  /**
   * @description 选择不保存时，如有复制按钮，则删除
   */
  notsave = () => {
    this.state.copyActions.forEach(item => {
      let _param = {
        func: 'sPC_MainMenu_Del',
        MenuID: item
      }
      Api.getCloudConfig(_param)
    })
    this.props.handleView()
  }

  /**
   * @description 编辑功能完成更新，包括解冻按钮、粘贴、替换等
   */
  editConfig = (res) => {
    this.setState({
      config: res.config
    })
  }

  /**
   * @description 更新搜索条件配置信息
   */
  updatesearch = (config) => {
    this.setState({
      config: config
    })
  }

  /**
   * @description 更新按钮配置信息
   */
  updateaction = (config, copyId, delcard) => {
    const { copyActions, delActions } = this.state

    this.setState({
      config: config,
      copyActions: copyId ? [...copyActions, copyId] : copyActions,
      delActions: delcard ? [...delActions, delcard] : delActions
    })
  }

  /**
   * @description 更新图表组配置信息
   */
  updatechartgroup = (config, _chartview) => {
    this.setState({
      config: config,
      chartview: _chartview
    })
  }
  
  /**
   * @description 更新配置信息
   */
  updateconfig = (config) => {
    this.setState({
      config: config
    })
  }

  refreshConfig = () => {
    const { menu } = this.props
    
    let param = {
      func: 'sPC_Get_LongParam',
      MenuID: menu.MenuID
    }

    Api.getCloudConfig(param).then(res => {
      if (res.status) {
        let _config = ''
        if (res.LongParam) {
          try {
            _config = JSON.parse(window.decodeURIComponent(window.atob(res.LongParam)))
          } catch (e) {
            console.warn('Parse Failure')
            _config = ''
          }
        }

        if (!_config) {
          notification.warning({
            top: 92,
            message: '未获取到配置信息！',
            duration: 5
          })
          return
        }

        _config.ParentId = menu.ParentId
        _config.fstMenuId = menu.fstMenuId
        _config.MenuName = menu.MenuName || ''
        _config.MenuNo = menu.MenuNo || ''
        _config.OpenType = menu.PageParam ? menu.PageParam.OpenType : ''
        _config.easyCode = _config.easyCode || ''

        // 版本兼容
        _config = updateCommonTable(_config)

        this.setState({
          config: null
        }, () => {
          this.setState({
            chartview: _config.charts ? _config.charts[0].uuid : '',
            config: _config,
            openEdition: res.open_edition || '',
            activeKey: menu.activeKey || '0',
            originActions: [],
            originMenu: fromJS(_config).toJS()
          })
        })
      } else {
        notification.warning({
          top: 92,
          message: res.message,
          duration: 5
        })
      }
    })
  }

  render () {
    const { menu } = this.props
    const { activeKey, config, chartview } = this.state

    if (!config) return null

    let configTabs = []
    config.tabgroups.forEach(group => {
      configTabs.push(...group.sublist)
    })

    return (
      <div className="common-table-board">
        <DndProvider backend={HTML5Backend}>
          {/* 工具栏 */}
          <div className="tools">
            <Collapse accordion activeKey={activeKey} bordered={false} onChange={(key) => this.setState({activeKey: key})}>
              {/* 基本信息 */}
              <Panel forceRender={true} header="基本信息" key="0" id="main-basedata">
                {/* 菜单信息 */}
                <MenuForm
                  menu={menu}
                  config={config}
                  updatemenu={this.updateconfig}
                />
                {config ? <UrlFieldComponent
                  config={config}
                  updateConfig={this.updateconfig}
                /> : null}
                {/* 表名添加 */}
                <TableComponent
                  config={config}
                  containerId="main-basedata"
                  updatetable={this.updateconfig}
                />
              </Panel>
            </Collapse>
          </div>
          <div className="setting">
            <Card title={
              <div>
                页面配置 
                <RedoOutlined style={{marginLeft: '10px'}} title="刷新标签列表" onClick={() => this.reloadTab(true)} />
              </div>
            } bordered={false} extra={
              <div>
                <Unattended config={config} updateConfig={this.updateconfig}/>
                {/* <Versions MenuId={menu.MenuID} open_edition={openEdition} updateConfig={this.refreshConfig}/> */}
                {/* <ReplaceField type="table" config={config} updateConfig={this.updateconfig}/> */}
                {/* <EditComponent type="table" options={['search', 'form', 'action', 'columns']} config={this.state.config} refresh={this.editConfig}/> */}
                <UpdateTable config={config}/>
                {/* <Switch className="big" checkedChildren="启" unCheckedChildren="停" checked={this.state.config.enabled} onChange={this.onEnabledChange} /> */}
                {/* <Button type="primary" id="save-config" onClick={this.submitConfig} loading={this.state.menuloading}>保存</Button> */}
                <Button onClick={this.cancelConfig}>关闭</Button>
              </div>
            } style={{ width: '100%' }}>
              <SettingComponent
                config={config}
                MenuID={menu.MenuID}
                updatesetting={this.updateconfig}
              />
              <SearchComponent
                config={config}
                updatesearch={this.updatesearch}
              />
              {config.charts ? <div className="chart-view" style={{position: 'relative'}}>
                {/* 视图组 已弃用 */}
                <ChartGroupComponent
                  config={config}
                  updatechartgroup={this.updatechartgroup}
                />
                {config.charts.map(item => {
                  if (!config.expand && chartview !== item.uuid) return ''

                  if (item.chartType === 'table') {
                    return (
                      <Col span={item.width || 24} key={item.uuid}>
                        {config.charts.length > 1 && item.title ? <p className="chart-title">{item.title}</p> : null}
                        <ActionComponent
                          type="main"
                          menu={{ MenuID: this.props.menu.MenuID, MenuName: config.MenuName, MenuNo: config.MenuNo, fstMenuList: this.props.menu.fstMenuList }}
                          config={config}
                          tabs={this.state.tabviews}
                          setSubConfig={(_btn) => this.setSubConfig(_btn, 'button')}
                          updateaction={this.updateaction}
                        />
                        <ColumnComponent
                          config={config}
                          menu={this.props.menu}
                          updatecolumn={this.updateconfig}
                        />
                      </Col>
                    )
                  } else if (item.chartType === 'card') {
                    return (
                      <Col span={item.width} key={item.uuid}>
                        <CardComponent
                          card={item}
                          config={config}
                          plotchange={this.updateconfig}
                        />
                      </Col>
                    )
                  } else {
                    return (
                      <Col span={item.width} key={item.uuid}>
                        <ChartComponent
                          plot={item}
                          config={config}
                          plotchange={this.updateconfig}
                        />
                      </Col>
                    )
                  }
                })}
              </div> : <>
                <ActionComponent
                  type="main"
                  menu={{ MenuID: this.props.menu.MenuID, MenuName: config.MenuName, MenuNo: config.MenuNo, fstMenuList: this.props.menu.fstMenuList }}
                  config={config}
                  tabs={this.state.tabviews}
                  setSubConfig={(_btn) => this.setSubConfig(_btn, 'button')}
                  updateaction={this.updateaction}
                />
                <ColumnComponent
                  config={config}
                  menu={this.props.menu}
                  updatecolumn={this.updateconfig}
                />
              </>}
              {/* 标签组 */}
              <TabsComponent
                config={config}
                tabs={this.state.tabviews}
                setSubConfig={(item) => this.setSubConfig(item, 'tab')}
                updatetabs={this.updateconfig}
              />
            </Card>
          </div>
        </DndProvider>
        {/* 返回时未保存提示 */}
        {/* <Modal
          bodyStyle={{textAlign: 'center', color: '#000000', fontSize: '16px'}}
          closable={false}
          maskClosable={false}
          visible={this.state.closeVisible}
          onCancel={() => { this.setState({closeVisible: false}) }}
          footer={[
            <Button key="save" className="mk-btn mk-green" loading={this.state.menucloseloading} onClick={this.submitConfig}>保存</Button>,
            <Button key="notsave" className="mk-btn mk-yellow" onClick={this.notsave}>不保存</Button>,
            <Button key="cancel" onClick={() => { this.setState({closeVisible: false}) }}>取消</Button>
          ]}
          destroyOnClose
        >
          配置已修改，是否保存配置信息？
        </Modal> */}
        {this.state.loading && <Spin size="large" />}
      </div>
    )
  }
}

export default ComTableConfig
